@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 use crate::exc::*;
-use crate::ffi::{PyDict_GET_SIZE, PyTypeObject};
+use crate::ffi::*;
 use crate::opt::*;
 use crate::serialize::bytes::*;
 use crate::serialize::dataclass::*;
@@ -16,7 +16,7 @@ use crate::serialize::tuple::*;
 use crate::serialize::uuid::*;
 use crate::serialize::writer::*;
 use crate::typeref::*;
-use serde::ser::{Serialize, SerializeMap, SerializeSeq, Serializer};
+use serde::ser::{Serialize, SerializeSeq, Serializer};
 use std::ptr::NonNull;
 
 pub const RECURSION_LIMIT: u8 = 255;
@@ -64,38 +64,36 @@ pub enum ObType {
 }
 
 pub fn pyobject_to_obtype(obj: *mut pyo3::ffi::PyObject, opts: Opt) -> ObType {
-    unsafe {
-        let ob_type = ob_type!(obj);
-        if ob_type == STR_TYPE {
-            ObType::Str
-        } else if ob_type == BYTES_TYPE {
-            ObType::Bytes
-        } else if ob_type == INT_TYPE
-            && (opts & PASSTHROUGH_BIG_INT == 0
-                || ffi!(_PyLong_NumBits(obj)) <= {
-                    if ffi!(Py_SIZE(obj)) > 0 {
-                        64
-                    } else {
-                        63
-                    }
-                })
-        {
-            ObType::Int
-        } else if ob_type == BOOL_TYPE {
-            ObType::Bool
-        } else if ob_type == NONE_TYPE {
-            ObType::None
-        } else if ob_type == FLOAT_TYPE {
-            ObType::Float
-        } else if ob_type == LIST_TYPE {
-            ObType::List
-        } else if ob_type == DICT_TYPE {
-            ObType::Dict
-        } else if ob_type == DATETIME_TYPE && opts & PASSTHROUGH_DATETIME == 0 {
-            ObType::Datetime
-        } else {
-            pyobject_to_obtype_unlikely(obj, opts)
-        }
+    let ob_type = ob_type!(obj);
+    if is_type!(ob_type, STR_TYPE) {
+        ObType::Str
+    } else if is_type!(ob_type, BYTES_TYPE) {
+        ObType::Bytes
+    } else if is_type!(ob_type, INT_TYPE)
+        && (opts & PASSTHROUGH_BIG_INT == 0
+            || ffi!(_PyLong_NumBits(obj)) <= {
+                if pylong_is_positive(obj) {
+                    64
+                } else {
+                    63
+                }
+            })
+    {
+        ObType::Int
+    } else if is_type!(ob_type, BOOL_TYPE) {
+        ObType::Bool
+    } else if is_type!(ob_type, NONE_TYPE) {
+        ObType::None
+    } else if is_type!(ob_type, FLOAT_TYPE) {
+        ObType::Float
+    } else if is_type!(ob_type, LIST_TYPE) {
+        ObType::List
+    } else if is_type!(ob_type, DICT_TYPE) {
+        ObType::Dict
+    } else if is_type!(ob_type, DATETIME_TYPE) && opts & PASSTHROUGH_DATETIME == 0 {
+        ObType::Datetime
+    } else {
+        pyobject_to_obtype_unlikely(obj, opts)
     }
 }
 
@@ -107,57 +105,49 @@ macro_rules! is_subclass {
 
 #[inline(never)]
 pub fn pyobject_to_obtype_unlikely(obj: *mut pyo3::ffi::PyObject, opts: Opt) -> ObType {
-    unsafe {
-        let ob_type = ob_type!(obj);
-        if ob_type == DATE_TYPE && opts & PASSTHROUGH_DATETIME == 0 {
-            ObType::Date
-        } else if ob_type == TIME_TYPE && opts & PASSTHROUGH_DATETIME == 0 {
-            ObType::Time
-        } else if ob_type == TUPLE_TYPE && opts & PASSTHROUGH_TUPLE == 0 {
-            ObType::Tuple
-        } else if ob_type == UUID_TYPE {
-            ObType::Uuid
-        } else if (*(ob_type as *mut PyTypeObject)).ob_type == ENUM_TYPE {
-            ObType::Enum
-        } else if opts & PASSTHROUGH_SUBCLASS == 0
-            && is_subclass!(ob_type, Py_TPFLAGS_UNICODE_SUBCLASS)
-        {
-            ObType::StrSubclass
-        } else if opts & PASSTHROUGH_SUBCLASS == 0
-            && is_subclass!(ob_type, Py_TPFLAGS_LONG_SUBCLASS)
-            && (opts & PASSTHROUGH_BIG_INT == 0
-                || ffi!(_PyLong_NumBits(obj)) <= {
-                    if ffi!(Py_SIZE(obj)) > 0 {
-                        64
-                    } else {
-                        63
-                    }
-                })
-        {
-            ObType::Int
-        } else if opts & PASSTHROUGH_SUBCLASS == 0
-            && is_subclass!(ob_type, Py_TPFLAGS_LIST_SUBCLASS)
-        {
-            ObType::List
-        } else if opts & PASSTHROUGH_SUBCLASS == 0
-            && is_subclass!(ob_type, Py_TPFLAGS_DICT_SUBCLASS)
-        {
-            ObType::Dict
-        } else if opts & PASSTHROUGH_DATACLASS == 0
-            && ffi!(PyDict_Contains((*ob_type).tp_dict, DATACLASS_FIELDS_STR)) == 1
-        {
-            ObType::Dataclass
-        } else if opts & SERIALIZE_NUMPY != 0 && is_numpy_scalar(ob_type) {
-            ObType::NumpyScalar
-        } else if opts & SERIALIZE_NUMPY != 0 && is_numpy_array(ob_type) {
-            ObType::NumpyArray
-        } else if opts & SERIALIZE_PYDANTIC != 0
-            && ffi!(PyDict_Contains((*ob_type).tp_dict, PYDANTIC_FIELDS_STR)) == 1
-        {
-            ObType::Pydantic
-        } else {
-            ObType::Unknown
-        }
+    let ob_type = ob_type!(obj);
+    if is_type!(ob_type, DATE_TYPE) && opts & PASSTHROUGH_DATETIME == 0 {
+        ObType::Date
+    } else if is_type!(ob_type, TIME_TYPE) && opts & PASSTHROUGH_DATETIME == 0 {
+        ObType::Time
+    } else if is_type!(ob_type, TUPLE_TYPE) && opts & PASSTHROUGH_TUPLE == 0 {
+        ObType::Tuple
+    } else if is_type!(ob_type, UUID_TYPE) {
+        ObType::Uuid
+    } else if is_type!(ob_type!(ob_type), ENUM_TYPE) {
+        ObType::Enum
+    } else if opts & PASSTHROUGH_SUBCLASS == 0 && is_subclass!(ob_type, Py_TPFLAGS_UNICODE_SUBCLASS)
+    {
+        ObType::StrSubclass
+    } else if opts & PASSTHROUGH_SUBCLASS == 0
+        && is_subclass!(ob_type, Py_TPFLAGS_LONG_SUBCLASS)
+        && (opts & PASSTHROUGH_BIG_INT == 0
+            || ffi!(_PyLong_NumBits(obj)) <= {
+                if pylong_is_positive(obj) {
+                    64
+                } else {
+                    63
+                }
+            })
+    {
+        ObType::Int
+    } else if opts & PASSTHROUGH_SUBCLASS == 0 && is_subclass!(ob_type, Py_TPFLAGS_LIST_SUBCLASS) {
+        ObType::List
+    } else if opts & PASSTHROUGH_SUBCLASS == 0 && is_subclass!(ob_type, Py_TPFLAGS_DICT_SUBCLASS) {
+        ObType::Dict
+    } else if opts & PASSTHROUGH_DATACLASS == 0 && pydict_contains!(ob_type, DATACLASS_FIELDS_STR) {
+        ObType::Dataclass
+    } else if opts & SERIALIZE_NUMPY != 0 && is_numpy_scalar(ob_type) {
+        ObType::NumpyScalar
+    } else if opts & SERIALIZE_NUMPY != 0 && is_numpy_array(ob_type) {
+        ObType::NumpyArray
+    } else if opts & SERIALIZE_PYDANTIC != 0
+        && (pydict_contains!(ob_type, PYDANTIC_FIELDS_STR)
+            || pydict_contains!(ob_type, PYDANTIC2_FIELDS_STR))
+    {
+        ObType::Pydantic
+    } else {
+        ObType::Unknown
     }
 }
 
@@ -213,27 +203,14 @@ impl Serialize for PyObjectSerializer {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(RECURSION_LIMIT_REACHED)
                 }
-                if unlikely!(unsafe { PyDict_GET_SIZE(self.ptr) } == 0) {
-                    serializer.serialize_map(Some(0)).unwrap().end()
-                } else if self.opts & NON_STR_KEYS == 0 {
-                    Dict::new(
-                        self.ptr,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
-                } else {
-                    DictNonStrKey::new(
-                        self.ptr,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
-                }
+                DictGenericSerializer::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                )
+                .serialize(serializer)
             }
             ObType::List => {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
@@ -264,31 +241,14 @@ impl Serialize for PyObjectSerializer {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
                     err!(RECURSION_LIMIT_REACHED)
                 }
-                let dict = ffi!(PyObject_GetAttr(self.ptr, DICT_STR));
-                let ob_type = ob_type!(self.ptr);
-                if unlikely!(
-                    dict.is_null() || ffi!(PyDict_Contains((*ob_type).tp_dict, SLOTS_STR)) == 1
-                ) {
-                    unsafe { pyo3::ffi::PyErr_Clear() };
-                    DataclassFallbackSerializer::new(
-                        self.ptr,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
-                } else {
-                    ffi!(Py_DECREF(dict));
-                    DataclassFastSerializer::new(
-                        dict,
-                        self.opts,
-                        self.default_calls,
-                        self.recursion,
-                        self.default,
-                    )
-                    .serialize(serializer)
-                }
+                DataclassGenericSerializer::new(
+                    self.ptr,
+                    self.opts,
+                    self.default_calls,
+                    self.recursion,
+                    self.default,
+                )
+                .serialize(serializer)
             }
             ObType::Pydantic => {
                 if unlikely!(self.recursion == RECURSION_LIMIT) {
