@@ -3,7 +3,7 @@
 use ahash::RandomState;
 use pyo3::ffi::*;
 use std::os::raw::c_char;
-use std::ptr::NonNull;
+use std::ptr::{null_mut, NonNull};
 use std::sync::Once;
 
 use crate::lazy::ThreadSafeLazy;
@@ -21,48 +21,49 @@ pub struct NumpyTypes {
     pub bool_: *mut PyTypeObject,
 }
 
-pub static mut NONE: *mut PyObject = 0 as *mut PyObject;
-pub static mut TRUE: *mut PyObject = 0 as *mut PyObject;
-pub static mut FALSE: *mut PyObject = 0 as *mut PyObject;
+pub static mut DEFAULT: *mut PyObject = null_mut();
+pub static mut OPTION: *mut PyObject = null_mut();
 
-pub static mut STR_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut INT_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut BOOL_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut NONE_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut FLOAT_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut LIST_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut DICT_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut DATETIME_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut DATE_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut TIME_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut TUPLE_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut UUID_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut ENUM_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
+pub static mut NONE: *mut PyObject = null_mut();
+pub static mut TRUE: *mut PyObject = null_mut();
+pub static mut FALSE: *mut PyObject = null_mut();
+pub static mut EMPTY_UNICODE: *mut PyObject = null_mut();
+
+pub static mut BYTES_TYPE: *mut PyTypeObject = null_mut();
+pub static mut BYTEARRAY_TYPE: *mut PyTypeObject = null_mut();
+pub static mut MEMORYVIEW_TYPE: *mut PyTypeObject = null_mut();
+pub static mut STR_TYPE: *mut PyTypeObject = null_mut();
+pub static mut INT_TYPE: *mut PyTypeObject = null_mut();
+pub static mut BOOL_TYPE: *mut PyTypeObject = null_mut();
+pub static mut NONE_TYPE: *mut PyTypeObject = null_mut();
+pub static mut FLOAT_TYPE: *mut PyTypeObject = null_mut();
+pub static mut LIST_TYPE: *mut PyTypeObject = null_mut();
+pub static mut DICT_TYPE: *mut PyTypeObject = null_mut();
+pub static mut DATETIME_TYPE: *mut PyTypeObject = null_mut();
+pub static mut DATE_TYPE: *mut PyTypeObject = null_mut();
+pub static mut TIME_TYPE: *mut PyTypeObject = null_mut();
+pub static mut TUPLE_TYPE: *mut PyTypeObject = null_mut();
+pub static mut UUID_TYPE: *mut PyTypeObject = null_mut();
+pub static mut ENUM_TYPE: *mut PyTypeObject = null_mut();
 pub static mut NUMPY_TYPES: ThreadSafeLazy<Option<NumpyTypes>> =
     ThreadSafeLazy::new(|| unsafe { load_numpy_types() });
 pub static mut FIELD_TYPE: ThreadSafeLazy<NonNull<PyObject>> =
     ThreadSafeLazy::new(|| unsafe { look_up_field_type() });
 
-pub static mut BYTES_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut BYTEARRAY_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
-pub static mut MEMORYVIEW_TYPE: *mut PyTypeObject = 0 as *mut PyTypeObject;
+pub static mut UTCOFFSET_METHOD_STR: *mut PyObject = null_mut();
+pub static mut NORMALIZE_METHOD_STR: *mut PyObject = null_mut();
+pub static mut CONVERT_METHOD_STR: *mut PyObject = null_mut();
+pub static mut DST_STR: *mut PyObject = null_mut();
 
-pub static mut INT_ATTR_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut UTCOFFSET_METHOD_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut NORMALIZE_METHOD_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut CONVERT_METHOD_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut EMPTY_UNICODE: *mut PyObject = 0 as *mut PyObject;
-pub static mut DST_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut DICT_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut DATACLASS_FIELDS_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut SLOTS_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut PYDANTIC_FIELDS_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut FIELD_TYPE_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut ARRAY_STRUCT_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut VALUE_STR: *mut PyObject = 0 as *mut PyObject;
-pub static mut STR_HASH_FUNCTION: Option<hashfunc> = None;
-pub static mut DEFAULT: *mut PyObject = 0 as *mut PyObject;
-pub static mut OPTION: *mut PyObject = 0 as *mut PyObject;
+pub static mut DICT_STR: *mut PyObject = null_mut();
+pub static mut DATACLASS_FIELDS_STR: *mut PyObject = null_mut();
+pub static mut SLOTS_STR: *mut PyObject = null_mut();
+pub static mut PYDANTIC_FIELDS_STR: *mut PyObject = null_mut();
+pub static mut PYDANTIC2_FIELDS_STR: *mut PyObject = null_mut();
+pub static mut FIELD_TYPE_STR: *mut PyObject = null_mut();
+pub static mut ARRAY_STRUCT_STR: *mut PyObject = null_mut();
+pub static mut VALUE_STR: *mut PyObject = null_mut();
+pub static mut INT_ATTR_STR: *mut PyObject = null_mut();
 
 pub static mut HASH_BUILDER: ThreadSafeLazy<ahash::RandomState> = ThreadSafeLazy::new(|| unsafe {
     RandomState::with_seeds(
@@ -74,9 +75,9 @@ pub static mut HASH_BUILDER: ThreadSafeLazy<ahash::RandomState> = ThreadSafeLazy
 });
 
 #[allow(non_upper_case_globals)]
-pub static mut MsgpackEncodeError: *mut PyObject = 0 as *mut PyObject;
+pub static mut MsgpackEncodeError: *mut PyObject = null_mut();
 #[allow(non_upper_case_globals)]
-pub static mut MsgpackDecodeError: *mut PyObject = 0 as *mut PyObject;
+pub static mut MsgpackDecodeError: *mut PyObject = null_mut();
 
 static INIT: Once = Once::new();
 
@@ -92,7 +93,6 @@ pub fn init_typerefs() {
         FALSE = Py_False();
         EMPTY_UNICODE = PyUnicode_New(0, 255);
         STR_TYPE = (*EMPTY_UNICODE).ob_type;
-        STR_HASH_FUNCTION = (*((*EMPTY_UNICODE).ob_type)).tp_hash;
         BYTES_TYPE = (*PyBytes_FromStringAndSize("".as_ptr() as *const c_char, 0)).ob_type;
 
         {
@@ -127,6 +127,8 @@ pub fn init_typerefs() {
             PyUnicode_InternFromString("__dataclass_fields__\0".as_ptr() as *const c_char);
         SLOTS_STR = PyUnicode_InternFromString("__slots__\0".as_ptr() as *const c_char);
         PYDANTIC_FIELDS_STR = PyUnicode_InternFromString("__fields__\0".as_ptr() as *const c_char);
+        PYDANTIC2_FIELDS_STR =
+            PyUnicode_InternFromString("model_fields\0".as_ptr() as *const c_char);
         FIELD_TYPE_STR = PyUnicode_InternFromString("_field_type\0".as_ptr() as *const c_char);
         ARRAY_STRUCT_STR =
             pyo3::ffi::PyUnicode_InternFromString("__array_struct__\0".as_ptr() as *const c_char);
@@ -140,7 +142,7 @@ pub fn init_typerefs() {
 
 #[cold]
 unsafe fn look_up_numpy_type(numpy_module: *mut PyObject, np_type: &str) -> *mut PyTypeObject {
-    let mod_dict = PyObject_GenericGetDict(numpy_module, std::ptr::null_mut());
+    let mod_dict = PyObject_GenericGetDict(numpy_module, null_mut());
     let ptr = PyMapping_GetItemString(mod_dict, np_type.as_ptr() as *const c_char);
     Py_XDECREF(ptr);
     Py_XDECREF(mod_dict);
@@ -174,7 +176,7 @@ unsafe fn load_numpy_types() -> Option<NumpyTypes> {
 #[cold]
 unsafe fn look_up_field_type() -> NonNull<PyObject> {
     let module = PyImport_ImportModule("dataclasses\0".as_ptr() as *const c_char);
-    let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
+    let module_dict = PyObject_GenericGetDict(module, null_mut());
     let ptr = PyMapping_GetItemString(module_dict, "_FIELD\0".as_ptr() as *const c_char)
         as *mut PyTypeObject;
     Py_DECREF(module_dict);
@@ -185,7 +187,7 @@ unsafe fn look_up_field_type() -> NonNull<PyObject> {
 #[cold]
 unsafe fn look_up_enum_type() -> *mut PyTypeObject {
     let module = PyImport_ImportModule("enum\0".as_ptr() as *const c_char);
-    let module_dict = PyObject_GenericGetDict(module, std::ptr::null_mut());
+    let module_dict = PyObject_GenericGetDict(module, null_mut());
     let ptr = PyMapping_GetItemString(module_dict, "EnumMeta\0".as_ptr() as *const c_char)
         as *mut PyTypeObject;
     Py_DECREF(module_dict);
@@ -196,7 +198,7 @@ unsafe fn look_up_enum_type() -> *mut PyTypeObject {
 #[cold]
 unsafe fn look_up_uuid_type() -> *mut PyTypeObject {
     let uuid_mod = PyImport_ImportModule("uuid\0".as_ptr() as *const c_char);
-    let uuid_mod_dict = PyObject_GenericGetDict(uuid_mod, std::ptr::null_mut());
+    let uuid_mod_dict = PyObject_GenericGetDict(uuid_mod, null_mut());
     let uuid = PyMapping_GetItemString(uuid_mod_dict, "NAMESPACE_DNS\0".as_ptr() as *const c_char);
     let ptr = (*uuid).ob_type;
     Py_DECREF(uuid);
